#!/usr/bin/env bash

set -e

install_cache_cert() {
    echo "${ITEM_PARAM_MCC_CA_CERT}" | base64 -d > "${CERT_FILE}"
}

put_nginx_config() {
    cat <<EOF > /etc/nginx/nginx.conf
user ${NGINX_USER};
events {}

http {
    server {
        listen 8801;
        location /deb/ {
            proxy_ssl_verify on;
            proxy_ssl_trusted_certificate ${CERT_FILE};
            proxy_pass ${ITEM_PARAM_MCP_APT_MIRROR}/ ;
        }
        location /bin/ {
            proxy_ssl_verify on;
            proxy_ssl_trusted_certificate ${CERT_FILE};
            proxy_pass ${ITEM_PARAM_MCP_BINARY_REGISTRY}/ ;
        }
    }
}
EOF
}

source vars.sh
source common.sh

if [ -e /etc/system-release ]; then
    OS_FAMILY="rhel"
else
    OS_FAMILY="debian"
fi

: "${ITEM_PARAM_MCP_APT_MIRROR:="https://mirror.mirantis.com"}"
: "${ITEM_PARAM_MCP_BINARY_REGISTRY:="https://binary.mirantis.com"}"
: "${ITEM_PARAM_PROXY_ENABLED:=${PROXY_ENABLED}}"

if ! timeout 1 bash -c 'cat < /dev/null > /dev/tcp/127.0.0.1/8801' &> /dev/null && [ "${ITEM_PARAM_PROXY_ENABLED}" == "true" ]; then
    # proxy enabled
    mkdir -p "${CERT_PATH}"
    if [ "$OS_FAMILY" == "debian" ]; then
        : "${ITEM_PARAM_KAAS_UBUNTU_REPO:=${KAAS_UBUNTU_REPO}}"

        UBUNTU_CODENAME='unknown'
        source <(cat /etc/os-release | grep UBUNTU_CODENAME)
        if [ "${UBUNTU_CODENAME}" == "unknown" ] && which lsb_release >/dev/null; then
            UBUNTU_CODENAME=$(lsb_release --codename --short)
        fi

        if [ "${UBUNTU_CODENAME}" != "bionic" ] && [ "${UBUNTU_CODENAME}" != "focal" ]; then
          echo "Unsupported ubuntu release codename: '${UBUNTU_CODENAME}'"
          exit 1
        fi

        export DEBIAN_FRONTEND=noninteractive
        export DEBCONF_NONINTERACTIVE_SEEN=true
        export APT_OPTS="-o Acquire::https::CAInfo=${CERT_FILE} -o APT::Install-Suggests=0 -o APT::Install-Recommends=0 -o Dpkg::Options::=--force-confold -o Dpkg::Options::=--force-confdef"
        NGINX_USER="www-data"

        # Remove default repositories
        find /etc/apt/ -name *.list -delete

        # Add Ubuntu mirrors
        cat <<EOF > /etc/apt/sources.list
# Ubuntu snapshot
deb [arch=amd64] ${ITEM_PARAM_MCP_APT_MIRROR}/${ITEM_PARAM_KAAS_UBUNTU_REPO} ${UBUNTU_CODENAME} main restricted universe
deb [arch=amd64] ${ITEM_PARAM_MCP_APT_MIRROR}/${ITEM_PARAM_KAAS_UBUNTU_REPO} ${UBUNTU_CODENAME}-updates main restricted universe
deb [arch=amd64] ${ITEM_PARAM_MCP_APT_MIRROR}/${ITEM_PARAM_KAAS_UBUNTU_REPO} ${UBUNTU_CODENAME}-security main restricted universe
EOF

        install_cache_cert
        # Install nginx
        apt_get_update
        apt-get install -y ${APT_OPTS} nginx
        put_nginx_config
        systemctl restart nginx
    else
        NGINX_USER="nginx"

        install_cache_cert
        source <(cat /etc/os-release | grep VERSION_ID)
        VERSION_ID="${VERSION_ID%%.*}"
        source <(cat /etc/os-release | grep -E "^ID=")
        PROXY="${HTTPS_PROXY:-${HTTP_PROXY}}"
        if [ "${ID}" == "centos" ] && [ -n "${PROXY}" ]; then
              sed -i '/#\?proxy \?=.*/d' /etc/yum.conf
              sed -i "/^\[main\]/a proxy=${PROXY}" /etc/yum.conf
        fi
        cat <<EOF > /etc/yum.repos.d/kubernetes_extra.repo
[kubernetes_extra]
name=Kubernetes Extra
baseurl=${ITEM_PARAM_MCP_APT_MIRROR}/kaas/kubernetes-extra-rhel${VERSION_ID}-0.0.5/
gpgcheck=0
enabled=1
proxy=_none_
sslcacert=${CERT_FILE}
EOF
        yum -y install yum-utils
        if [ "${ID}" == "rhel" ] && [ "${VERSION_ID}" == "7" ]; then
            yum-config-manager --enable \
            rhui-rhel-${VERSION_ID}-server-rhui-extras-rpms \
            rhel-${VERSION_ID}-server-extras-rpms
        fi
        if [ "${VERSION_ID}" == "8" ]; then
            yum -y install policycoreutils-python-utils
        fi
        yum -y install nginx
        if systemctl is-active firewalld &>/dev/null ; then
            firewall-cmd --permanent --add-port=8801/tcp
        fi
        if [ "$(getenforce)" != "Disabled" ]; then
            # semanage port add will fail if port exists
            semanage port -a -t http_port_t -p tcp 8801 || semanage port -m -t http_port_t -p tcp 8801
            # Allow nginx to connect to upstream
            setsebool httpd_can_network_connect on -P
        fi
        put_nginx_config
        systemctl restart nginx
    fi
fi
